import json
import os

from gbdx_task_template.test.test_base import TestBase
from gbdx_task_template.task import Task


class TestTask(TestBase):

    def test_bad_port_obj(self):
        try:
            self.task.bad_port = "Not a Port"
        except TypeError as e:
            assert "Task ports must be of type Port" == e.message

    def test_task_adders(self):
        new_task = Task(
            "ThisTask",
            timeout=1,
            callback='my_func',
            impersonation_allowed=True
        )
        task_json = json.loads(new_task.json())
        assert task_json['timeout'] == 1
        assert task_json['callback'] == 'my_func'
        assert task_json['impersonation_allowed'] is True

    def test_task_json(self):
        print('=====', os.environ.get('REMOTE_WORK_PATH'))
        task_wf = json.loads(self.task.json())
        assert self.task.type.lower() in task_wf['name']
        assert self.task.type == task_wf['taskType']

        # Check Ports
        self.assertEqual(len(task_wf['inputs']), 3)
        for port in task_wf['inputs']:
            self.assertIn(port['name'], self.basic_inport_names)

        self.assertEqual(len(task_wf['outputs']), 1)
        for port in task_wf['outputs']:
            self.assertIn(port['name'], self.basic_outport_names)

    def test_task_validate_local(self):
        print('=====', os.environ.get('REMOTE_WORK_PATH'))
        self.assertTrue(self.task.is_valid())



import os
import json
import uuid

from gbdx_task_template.port import Port, InputPort


class Task(object):
    """
    Class to encapsulate the Task configuration
    """

    def __init__(self, name, **kwargs):
        self._task_name = name
        self._task_type = "CloudHarness_Anonymous_Task"

        # JSON for this task only.
        self._task_wf_json = {
            "inputs": [],
            "outputs": [],
            "properties": {"timeout": 7200},
            "taskType": self._task_type,
            "name": '%s_%s' % (self._task_type.lower(), str(uuid.uuid4()))
        }

        if 'timeout' in kwargs.keys():
            self._task_wf_json['timeout'] = kwargs['timeout']

        if 'impersonation_allowed' in kwargs.keys():
            self._task_wf_json['impersonation_allowed'] = kwargs['impersonation_allowed']

        if 'callback' in kwargs.keys():
            self._task_wf_json['callback'] = kwargs['callback']

        # Add default source bundle port
        port_name = "source_bundle"
        dest_path = os.path.join(os.getcwd(), port_name)
        srcbundle = InputPort(value=dest_path)
        srcbundle.name = port_name
        srcbundle.multiplex = False
        self.source_bundle = srcbundle

    def __setattr__(self, key, value):
        # Check if key already exists
        if Port.is_port(value):
            value.name = key
            self.add_port(value)

        super(Task, self).__setattr__(key, value)

    @property
    def definition(self):
        filename = os.path.join(
            os.path.dirname(os.path.abspath(__file__)),
            'task_definition.json'
        )

        with open(filename, 'r') as f:
            return json.loads(f.read())

    @property
    def src_path(self):
        return self.source_bundle.value

    @property
    def type(self):
        return self._task_type

    @property
    def name(self):
        return self._task_name

    @name.setter
    def name(self, value):
        self._task_name = value

    @property
    def input_ports(self):
        return self._task_wf_json['inputs']

    @property
    def output_ports(self):
        return self._task_wf_json['outputs']

    @property
    def ports(self):
        return self._task_wf_json['inputs'], self._task_wf_json['outputs']

    def json(self):
        return json.dumps(self._task_wf_json, cls=CustomEncoder, indent=2)

    def add_port(self, new_port):
        if not Port.is_port(new_port):
            raise TypeError("Task ports must be of type Port")

        if new_port.direction == 'Input':
            self._task_wf_json['inputs'].append(new_port)
        elif new_port.direction == 'Output':
            self._task_wf_json['outputs'].append(new_port)

    def is_valid(self, remote=False):
        """
        Check cloud-harness code is valid. task schema validation is
        left to the API endpoint.
        :param remote: Flag indicating if the task is being ran on the platform or not.
        :return: is valid or not.
        """

        if len(self.input_ports) < 1:
            return False

        if remote:
            # Ignore output ports as value will overriden.
            ports = [
                port for port in self.input_ports if port.type == 'directory'
                ]
            for port in ports:
                # Will raise exception if the port is invalid.
                port.is_valid_s3_url(port.value)
        else:
            all_ports = self.ports[0] + self.ports[1]
            ports = [
                port for port in all_ports if port.type == 'directory' and port.name != 'source_bundle'
                ]
            for port in ports:
                # Will raise exception if the port is invalid.
                port.is_valid_filesys(port.value)

        return True


class CustomEncoder(json.JSONEncoder):
    def default(self, obj):
        # For Port instances
        if isinstance(obj, Port):
            return obj.json
        # Let the base class default method raise the TypeError
        return json.JSONEncoder.default(self, obj)



import logging
import sys


def get_logger(name, level=0):
    logger = logging.getLogger(name)

    handler = logging.StreamHandler(sys.stdout)
    handler.setLevel(level)

    formatter = logging.Formatter(
            '%(asctime)s %(name)-12s %(levelname)-8s %(message)s')
    handler.setFormatter(formatter)

    logger.addHandler(handler)

    logger.setLevel(level)

    return logger


import os
from unittest import TestCase
from moto import mock_s3

from gbdx_cloud_harness.services.account_storage_service import AccountStorageService


class TestPortService(TestCase):

    def setUp(self):
        self.mock = mock_s3()
        self.mock.start()

        self.test_bucket = 'bckt_not_provided'
        self.test_account_id = 'pre_not_provided'

        self.test_storage = AccountStorageService(is_test=True)

    def tearDown(self):
        self.mock.stop()

    def test_location(self):
        loc_path = self.test_storage.location
        assert self.test_bucket in loc_path
        assert self.test_account_id in loc_path

    def test_upload_file(self):
        upload_file = os.path.join(
            os.path.dirname(os.path.abspath(__file__)),
            'input', 'app.py'
        )
        s3_path = 'input/app.py'
        self.test_storage.upload((upload_file, s3_path))

        bucket_contents = self.test_storage.client.list_objects(Bucket=self.test_bucket)['Contents']

        assert bucket_contents[0]['Key'] == '%s/%s' % (self.test_account_id, s3_path)

        acct_contents = self.test_storage.list()

        assert acct_contents[0] == '%s/%s' % (self.test_account_id, s3_path)

        upload_file = os.path.join(
            os.path.dirname(os.path.abspath(__file__)),
            'input', 'app.py'
        )
        self.test_storage.upload((upload_file, 'app.py'))

        acct_contents = self.test_storage.list()
        acct_contents_input = self.test_storage.list(s3_folder='input')

        # assert only one file was listed.
        assert len(acct_contents) == 2
        assert acct_contents_input[0] == '%s/%s' % (self.test_account_id, s3_path)